/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.core.engine;

import java.lang.annotation.Annotation;

import net.lucubrators.honeycomb.core.controller.FormObjectHolder;
import net.lucubrators.honeycomb.core.controller.FormObjectProvider;
import net.lucubrators.honeycomb.core.controller.HandleEvent;
import net.lucubrators.honeycomb.core.controller.Populate;
import net.lucubrators.honeycomb.core.reflect.exceptions.CombInvocationException;

/**
 * Interface representing the caller of controller methods.
 * 
 * @author gnalFF
 * 
 */
public interface ControllerInvoker {

	/**
	 * Method should invoke the {@link Populate} annotated method of a
	 * controller.
	 * 
	 * @param controller
	 *            the controller holding the method.
	 * @param bindables
	 *            the current {@link Bindables} object
	 * @return the result of the method invocation.
	 * @throws CombInvocationException
	 *             will be thrown if the invoked method fails. The cause of this
	 *             exception should represent the reason of failing.
	 */
	Object doGet(Object controller, Bindables bindables) throws CombInvocationException;

	/**
	 * Method should invoke the {@link HandleEvent} annotated method of a
	 * controller.
	 * 
	 * @param controller
	 *            the controller object holding the method.
	 * @param bindables
	 *            the current {@link Bindables} object.
	 * @return the result of the method invocation.
	 * @throws CombInvocationException
	 *             will be thrown if the invoked method fails. The cause of this
	 *             exception should represent the reason of failing.
	 */
	Object doPost(Object controller, Bindables bindables) throws CombInvocationException;

	/**
	 * Method should invoke the {@link FormObjectProvider} annotated method of a
	 * controller.
	 * 
	 * @param controller
	 *            the controller object holding the method.
	 * @param bindables
	 *            the current {@link Bindables} object
	 * @return a {@link FormObjectHolder} holding all the needed attributes
	 *         provided by the annation and of course the result of the
	 *         invocation.
	 * @throws CombInvocationException
	 *             will be thrown if the invoked method fails. The cause of this
	 *             exception should represent the reason of failing.
	 */
	FormObjectHolder getFormObject(Object controller, Bindables bindables) throws CombInvocationException;

	/**
	 * Checks if the given controller object contains a method annotated with
	 * the given annotation type.
	 * 
	 * @param controller
	 *            the controller object to be checked for the annotated method.
	 * @param annotation
	 *            the annotation type to be looked for.
	 * @return true if there is a method having the annotation.
	 */
	boolean hasMethodAnnotatedWith(Object controller, Class<? extends Annotation> annotation);

}