/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.core.messages;

/**
 * Message indicating that the user made a mistake on form inputs.
 * Extend this class if want to add additional behaviour like getting the
 * message from a resource bundle.
 * 
 * @author aglomann
 * 
 */
public final class ErrorMessage extends Message {

	private final String	property;
	private final Object	errorValue;
	private final boolean	errorValueSet;

	/**
	 * Constructor, implicitly {@link MessageLevel#ERROR} will be set
	 * 
	 * @param property
	 *            the property of a form object which caused the error
	 * @param message
	 *            the description of the error
	 */
	public ErrorMessage(final String property, final String message) {
		super(message, MessageLevel.ERROR);
		this.property = property;
		errorValueSet = false;
		errorValue = null;
	}

	/**
	 * Constructor, implicitly {@link MessageLevel#ERROR} will be set
	 * 
	 * @param property
	 *            the property of a form object which cause the error
	 * @param message
	 *            the description of the error
	 * @param errorValue
	 *            the value which caused the error
	 */
	public ErrorMessage(final String property, final String message, final Object errorValue) {
		super(message, MessageLevel.ERROR);
		this.property = property;
		errorValueSet = true;
		this.errorValue = errorValue;
	}

	public String getProperty() {
		return property;
	}

	public boolean hasErrorValue() {
		return errorValueSet;
	}

	public Object getErrorValue() {
		return errorValue;
	}

	@Override
	public String toString() {
		return "ErrorMessage [property=" + property + ", errorValue=" + errorValue + ", errorValueSet=" + errorValueSet + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((errorValue == null) ? 0 : errorValue.hashCode());
		result = prime * result + (errorValueSet ? 1231 : 1237);
		result = prime * result + ((property == null) ? 0 : property.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		ErrorMessage other = (ErrorMessage) obj;
		if (errorValue == null) {
			if (other.errorValue != null)
				return false;
		} else if (!errorValue.equals(other.errorValue))
			return false;
		if (errorValueSet != other.errorValueSet)
			return false;
		if (property == null) {
			if (other.property != null)
				return false;
		} else if (!property.equals(other.property))
			return false;
		return true;
	}

}
